// Generated by CoffeeScript 1.8.0
(function() {
  var components, decompose, hintRegex, parse, phoneUtil, resolve, stripRegex, stripType, supportedRegionCodes;

  phoneUtil = require('libphonenumber').phoneUtil;

  supportedRegionCodes = ['US', 'CA', 'GB'];

  parse = function(string, req) {
    var number, parts, regionCode, type, _ref, _ref1;
    if (string == null) {
      return string;
    }
    _ref = stripType(string), type = _ref[0], string = _ref[1];
    _ref1 = resolve(string, req != null ? req.logger : void 0), number = _ref1[0], regionCode = _ref1[1];
    if (number) {
      parts = decompose(string, number, regionCode);
      parts.type = type;
      return parts;
    } else {
      parts = new String(string);
      parts.raw = string;
      parts.type = type;
      return parts;
    }
  };

  hintRegex = /[(]?([hwcm])[)]?$/;

  stripRegex = /^\s+|\s+$/g;

  stripType = function(string) {
    var match, type;
    match = string.match(hintRegex);
    if (match) {
      type = (function() {
        switch (match[1]) {
          case 'h':
            return 'home';
          case 'w':
            return 'work';
          case 'c':
          case 'm':
            return 'mobile';
        }
      })();
      return [type, string.replace(hintRegex, '').replace(stripRegex, '')];
    } else {
      return [null, string];
    }
  };

  resolve = function(string, logger) {
    var e, number, regionCode, _i, _len;
    number = null;
    for (_i = 0, _len = supportedRegionCodes.length; _i < _len; _i++) {
      regionCode = supportedRegionCodes[_i];
      try {
        number = phoneUtil.parse(string, regionCode);
      } catch (_error) {
        e = _error;
        if (logger != null) {
          if (typeof logger.error === "function") {
            logger.error(e);
          }
        }
        number = null;
      }
      if (number != null) {
        break;
      }
    }
    if (number) {
      return [number, regionCode];
    } else {
      return [null, null];
    }
  };

  decompose = function(raw, number, regionCode) {
    var exchange, extension, line, nationalDestinationCode, nationalDestinationCodeLength, nationalPrefix, nationalSignificantNumber, phone, subscriberNumber;
    nationalPrefix = phoneUtil.getNddPrefixForRegion(regionCode, true);
    nationalSignificantNumber = phoneUtil.getNationalSignificantNumber(number);
    nationalDestinationCodeLength = phoneUtil.getLengthOfNationalDestinationCode(number);
    if (nationalDestinationCodeLength > 0) {
      nationalDestinationCode = nationalSignificantNumber.substring(0, nationalDestinationCodeLength);
      subscriberNumber = nationalSignificantNumber.substring(nationalDestinationCodeLength);
    } else {
      nationalDestinationCode = null;
      subscriberNumber = nationalSignificantNumber.substring(nationalDestinationCodeLength);
    }
    extension = number.getExtension();
    if (['US', 'CA'].indexOf(regionCode) !== -1) {
      exchange = subscriberNumber.substring(0, 3);
      line = subscriberNumber.substring(3);
    } else {
      null;
    }
    phone = new String(nationalSignificantNumber || raw);
    phone.prefix = nationalPrefix;
    phone.raw = raw;
    phone.area = nationalDestinationCode;
    phone.exchange = exchange;
    phone.line = line;
    phone.number = subscriberNumber;
    phone.extension = extension;
    phone.country_code = regionCode;
    return phone;
  };

  components = [
    {
      name: 'raw',
      type: 'string',
      description: 'Unmodified value'
    }, {
      name: 'area',
      type: 'string',
      description: 'Area code'
    }, {
      name: 'exchange',
      type: 'string',
      description: 'Exchange'
    }, {
      name: 'line',
      type: 'string',
      description: 'Line'
    }, {
      name: 'number',
      type: 'string',
      description: 'Full number'
    }, {
      name: 'extension',
      type: 'string',
      description: 'Extension'
    }, {
      name: 'country_code',
      type: 'string',
      description: 'Country code'
    }, {
      name: 'type',
      type: 'string',
      description: 'Number type: home, work, mobile'
    }
  ];

  module.exports = {
    parse: parse,
    components: components,
    countryCodes: supportedRegionCodes,
    maskable: false
  };

}).call(this);
